/*
Timer for UV bed. Time can be set with switches on PBX and X, exposure can be
paused and resumed with PBX. A6810 is used to interface with VFD. A6810 is
controlled by PORTA. Time is retreived from PIC's internal EEPROM. Clock = 4MHz
Inputs are active low.
*/

// Define outputs
#define OUT PORTA.B0
#define STROBE PORTA.B3
#define CLK PORTA.B2
#define UV PORTB.B5

// Define inputs
#define PAUSE PORTB.B0
#define TUP PORTB.B2
#define TDOWN PORTB.B1

#define TMRSET 19

     unsigned time;
     char counter, hundred, ten, one, rbint;

// Handles interrupts
void interrupt() {
     // Timer0 interrupt
  if (T0IF_bit){
     counter++;
     TMR0 = TMRSET;
     T0IF_bit = 0;
  }
  // RB0 interrupt
  if (INTF_bit){
     INTE_bit = 0;
     // Toggle Timer enable, RB interrupt flag and UV.
     T0IE_bit = !T0IE_bit;
     rbint = !rbint;
     UV = !UV;
     // debounce delay
     Delay_ms(30);
     // renable RB interrupts
     INTF_bit = 0;
     INTE_bit = 1;
  }
}

// writes a digit and grid index to A6810.
void write(unsigned temp, char grid){
     char i;

     // Set correct digit pattern
     switch (temp){
            case 0 : temp = 63; break;
            case 1 : temp = 6; break;
            case 2 : temp = 91; break;
            case 3 : temp = 79; break;
            case 4 : temp = 102; break;
            case 5 : temp = 109; break;
            case 6 : temp = 125; break;
            case 7 : temp = 7; break;
            case 8 : temp = 127; break;
            case 9 : temp = 111; break;
            default : temp = 64;
     }

     // Select correct grid
     switch (grid){
            case 0 : temp = temp | 0b0010000000; break;
            case 1 : temp = temp | 0b0100000000; break;

            // Special condition to set dots
            case 2 : temp = temp | 0b0000110000; break;

            case 3 : temp = temp | 0b1000000000; break;
            default : temp = temp | 0b1111000000;
     }

     // Clock values into A6810's shift register
     for (i=0; i<10; i++){
         OUT = temp;
         CLK = 1;
         CLK = 0;
         temp = temp >> 1;
     }

     // Set the contents of the register to the outputs.
     STROBE = 1;
     STROBE = 0;
}

// gets the time stored in the EEPROM
void gettime(){
     char LSB, MSB;
     LSB = EEPROM_Read(0x01);
     Delay_ms(25);
     MSB = EEPROM_Read(0x00);
     time = (unsigned)(MSB<<8) | (unsigned)(LSB);
     Delay_ms(25);
}

// Saves the current time to the EEPROM
void savetime(){
     EEPROM_Write(0x01, time);
     Delay_ms(25);
     EEPROM_Write(0x00, (time>>8));
     Delay_ms(25);
}

// convert time to M:SS format and write time to VFD display
void display(){
     one = time % 10;
     write(one, 0);

     ten = time / 10;
     hundred = ten / 10;
     ten = ten % 10;
     write(ten, 1);
     // slight delay so each segment is lit for an equal amount of time
     Delay_us(250);

     // write dots
     write(0, 2);
     write(hundred, 3);
}

// Main
void main() {

     char loop, oldcount;

     TRISA = 0;               // set direction to be output
     TRISB = 0b11011111;      // set direction to be input for all save RB5 (UV control)
     PORTA = 0;               // Initialize PORTA

     OPTION_REG = 0b00000111; // Set 1:256 prescaler on timer 0 and enable portb pull-up resistors
     
     UV = 0;                  // Ensure that UV is off
     
     // gets time saved in EEPROM from previous use
     gettime();
     if (time>959) time = 959;

     Delay_ms(1000);          // Start-up delay, wait for supply voltages to rise
     counter = 0;
     rbint = 1;
     oldcount = 0;
     
     GIE_bit = 1;             // Allow timer0 and RB0 interrupts
     T0IE_bit = 1;
     INTE_bit = 1;
     
     // Start-up "menu", allows change of time
     while (rbint){
     
           display();
           
           // increment time
           if ((!TUP) && (oldcount < counter)){
              // set current counter value to oldcount, used to set debounce timeout
              oldcount = counter+1;
              if (oldcount > 253) oldcount = TMRSET;
              // check if next number should be 00 instead of 60
              if ((ten==5) && (one == 9)){
                    // check if at 9:59, set to zero
                    if (hundred == 9){
                       time = 0;
                    }else{
                          time++;
                          time = time + 40;
                    }
                 }else{
                       time++;
                 }
           }
           
           // decrement time
           if ((!TDOWN) && (oldcount < counter)){
              // set current counter value to oldcount, used to set debounce timeout
              oldcount = counter+1;
              if (oldcount > 253) oldcount = TMRSET;
              // check if next number should be 59 instead of 99
              if ((ten==0) && (one == 0)){
                 // check if at zero, set to max again
                 if (hundred == 0){
                    time = 959;
                 }else{
                       time--;
                       time = time - 40;
                 }
              }else{
                    time--;
              }
           }
           
     }
     
     // saves settings and starts exposure
     time = one + (10*ten) + (100*hundred);
     savetime();
     loop = 1;
     T0IE_bit = 1;

     while (loop){
           
           display();

           // tracks time based on Timer0 interrupts
           if (counter > 16){
                 counter = 0;
                 // check if next number should be 59 instead of 99
                 if ((ten==0) && (one == 0)){
                    // check if done counting
                    if (hundred == 0){
                       loop = 0;
                    }else{
                          time--;
                          time = time - 40;
                    }
                 }else{
                       time--;
                 }
           }
           
           while (rbint){
           display();
           }
     }

     //timer finished, turn off UV LEDs and set VFD to 0:00
     
     GIE_bit = 0;      // Disable interrupts
     UV = 0;

     while (1){
           write(0, 0);
           write(0, 1);
           write(0, 2);
           write(0, 3);
     }
}
